/***********************************************************
PROJECT  : DS1621 (on I2c) based thermometer
MODULE   : Thermometer querying
TARGET   : Linux/x86 + Bit-banging or PCF8584 driver
COPYRIGHT: (c) 2002 Damien Mercier <Damien.Mercier@grosmi.net>
LICENCE  : LGPL
CREATION : Sun Feb 11 16:34:22 MET 2001 on Idefix
UPDATE   : Wed Apr 10 14:20:06 CEST 2002 on Idefix
VERSION  : 1.3
***********************************************************/
/*
This code is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This code is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/
/* works with all the Yail-I2C library/drivers */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "getopt.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <signal.h> 
#include <fcntl.h> 

#include "libi2c.h"

#define FRACTIONNAIRE

/* Lower bound for variations */
#define EPSIL 20

/* this is the default address */
#define DS1621_ADR 0x9E

#define CONV_DELAY 500

char  *usage_string[] = {
"Thermometer based on DS1621 i2c chip",
"Copyright (c) 2000-2002 Damien Mercier (LGPL)",
"Options :",
"          -h : help",
"          -l : human readable values",
"  -p <delay> : periodical reading (seconds)",
"          -i : Asymptotical/Slope extrapolation (implies -p)",
"-a <address> : DS1621 address (default 0x9E)",
"   -t <time> : conversion time (default 500ms)",
" <no options>: temperature shown as 2036 for 20.36 degree C",
NULL};

/* Help */
void Usage(char *argv[])
{
   extern char *usage_string[];
   int i;

   printf("** %s **:\n",argv[0]);
   i = 0;
   while (usage_string[i])
      	printf("%s\n",usage_string[i++]);
   exit(-1);
}

int median(int a, int b, int c) {
    int t;
    if (a>b) {
        t=a;a=b;b=t;
    }
    if (b<c) return b;
    if (c<a) return a;
    return c;
}

int main(int argc, char **argv) {
  u_char adth;
  long delay;
  int longdisp,period;
  int count,slope,temp;
  char buf[256];
  int k;
  int t1,t2,t3,te1,te2,te3;
  int te,tendancy,extrap;
  char cc;

  /* DS1621 address */
  adth=DS1621_ADR;
  delay=CONV_DELAY*1000;
  longdisp=0;period=0;extrap=0;
  t1=t2=t3=te1=te2=te3=te=2000;cc=' ';

  while (1) {
    k=getopt(argc,argv,"hlip:a:t:");
    if (k==-1) break;
    switch (k) {
        case 'h':
            Usage(argv);
            break;
        case 'l':
            longdisp=1;
            break;
        case 'p':
            period=atol(optarg);
            break;
        case 'i':
            extrap=1;
            break;
        case 't':
            delay=atol(optarg)*1000;
            break;
        case 'a':
            adth=(u_char)strtol(optarg,NULL,16);
            break;
    }
  }
  
  if ((extrap == 1) && (period < 10)) {
    printf("-i option implies -p val with val>=10s!\n");
    Usage(argv);
  }

  if (i2c_init(0)) {
   fprintf(stderr,"Unable to initialize i2c bus\n");
   exit(-1);
  }
    
  /* DS1621 setup */
  buf[0]=0xAC;
  buf[1]=1;
  i2c_write(adth,buf,2);  
  if (longdisp) {
    printf("Thermometer based on DS1621 i2c chip\n");
    printf("Copyright (c) 2000-2002 Damien Mercier (LGPL)\n");
  }
  /* start temperature conversion */
  buf[0]=0xEE;
  i2c_write(adth,buf,1);
  usleep(delay); /* conversion delay */
  do {
    /* stop conversion */
    buf[0]=0x22;
    i2c_write(adth,buf,1);
    /* Temperature reading (1 Celsius degree precision) */
    buf[0]=0xAA;
    i2c_write_read(adth,buf,1,buf,1);
    temp=buf[0];
    /* Counter reading (fraction of degree) ) */
    buf[0]=0xA8;
    i2c_write_read(adth,buf,1,buf,1);
    count=buf[0];
    /* slope reading */
    buf[0]=0xA9;
   i2c_write_read(adth,buf,1,buf,1);
   slope=buf[0];
   k=temp;
   if (slope != 0) {
       k=temp*100-25+(100*(slope-count))/slope;
   }
   if (extrap) {
       t1=t2;t2=t3;t3=k;
       /* extrapolated from 3 equidistant measures */
       tendancy=2*t2-t1-t3;
       if (tendancy==0) {
            te3=t3;
       } else {
            te3=(t2*t2-t1*t3)/tendancy;
       }
       cc=(tendancy>EPSIL)?'+':((tendancy < -EPSIL)?'-':'=');
       if (tendancy*(t3-t1) < -2*EPSIL) {
        /* temperature quick change, no estimation */
        te3=t3;
        cc='*';
       }
       /* Now median of 3 last predictions */
       te=median(te1,te2,te3);
       te1=te2;te2=te3;
       
   }
   if (!longdisp) {
    if (extrap) {
        printf("%d  %d  %c\n",k,te,cc);
    } else {
        printf("%d\n",k);
    }
   } else {
        /* printf("Temperature : %d degree C\n",temp); */
        printf("Temperature : %2.2f degree C\n",(float)k/100);
    if (extrap) {
        printf("Extrapolated asymptotical equilibrium %2.2f degree C\n",(float)te/100);
        printf("Evolution : %c\n",cc);
    }
    printf("\n");
   }
    
    if (period>0) {
        /* start temperature conversion */
        buf[0]=0xEE;
        i2c_write(adth,buf,1);
        sleep(period);
    }
  } while (period>0);
  i2c_close();
  exit(0);
}
